package org.jeecg.modules.adweb.dmp.service.impl;

import static org.jeecg.modules.adweb.dmp.vo.report.SiteOverviewStatsVO.DailyStatsVO;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Sets;

import org.apache.commons.compress.utils.Lists;
import org.jeecg.common.util.DateUtils;
import org.jeecg.modules.adweb.common.util.DateUtil;
import org.jeecg.modules.adweb.dmp.entity.GADailyReport;
import org.jeecg.modules.adweb.dmp.mapper.GADailyReportMapper;
import org.jeecg.modules.adweb.dmp.service.IGADailyReportService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description: dmp_ga_daily_report @Author: jeecg-boot @Date: 2024-11-04 @Version: V1.0
 */
@Service
public class GADailyReportServiceImpl extends ServiceImpl<GADailyReportMapper, GADailyReport>
        implements IGADailyReportService {

    @Autowired private GADailyReportMapper gaDailyReportMapper;

    /**
     * 查询指定时间区间内的{@DailyStatsVO}
     *
     * <p>如数据库中某天数据缺失，需填充空数据
     */
    public List<DailyStatsVO> getDailyStatsWithinPeriod(String siteCode, Date start, Date end) {
        List<DailyStatsVO> dailyStatsVOs = Lists.newArrayList();

        List<GADailyReport> gaDailyReports =
                this.list(
                        new LambdaQueryWrapper<GADailyReport>()
                                .eq(GADailyReport::getSiteCode, siteCode)
                                .ge(GADailyReport::getDate, start)
                                .le(GADailyReport::getDate, end)
                                .orderByAsc(GADailyReport::getDate));

        // 1. VO数据填充
        for (GADailyReport gaDailyReport : gaDailyReports) {
            DailyStatsVO dailyStatsVO = new DailyStatsVO();
            dailyStatsVO.setDate(
                    DateUtils.date2Str(gaDailyReport.getDate(), DateUtils.date_sdf.get()));
            dailyStatsVO.setTotalUsers(gaDailyReport.getTotalUsers());
            dailyStatsVO.setSessions(gaDailyReport.getSessions());
            dailyStatsVO.setBounceRate(gaDailyReport.getBounceRate());
            dailyStatsVO.setAvgSessionDuration(gaDailyReport.getAvgSessionDuration());
            dailyStatsVO.setPageViews(gaDailyReport.getPageViews());
            dailyStatsVO.setPageViewsPerSession(gaDailyReport.getPageViewsPerSession());

            // TODO: 待后续步骤设置
            dailyStatsVO.setEnquires(0);

            dailyStatsVOs.add(dailyStatsVO);
        }

        // 2. 缺失日期补充
        Set<String> absentDays =
                Sets.difference(
                        DateUtil.getAllDaysBetween(start, end).stream()
                                .map(date -> DateUtils.date2Str(date, DateUtils.date_sdf.get()))
                                .collect(Collectors.toSet()),
                        dailyStatsVOs.stream()
                                .map(DailyStatsVO::getDate)
                                .collect(Collectors.toSet()));
        absentDays.forEach(
                date -> {
                    DailyStatsVO dailyStatsVO = new DailyStatsVO();
                    dailyStatsVO.setDate(date);
                    // 全部设置为默认值
                    dailyStatsVO.setTotalUsers(0);
                    dailyStatsVO.setPageViews(0);
                    dailyStatsVO.setEnquires(0);

                    dailyStatsVOs.add(dailyStatsVO);
                });

        // 3. 根据日期排序并返回
        return dailyStatsVOs.stream()
                .sorted(Comparator.comparing(DailyStatsVO::getDate)) // 排日期升序排序
                .toList();
    }
}
